#!/bin/bash
#
#    Behera, S. K., J. Luo, S. Masson, S. Rao, S. Gualdi, P. Delecluse, A.
#    Navarra and T. Yamagata, 2004 : Paramount Impact of the Indian Ocean
#    Dipole on the East African Short Rains: A CGCM Study, J. Climate, In
#    press.
#
#    donnerait
#
#    <biblioentry xml:id="behara2004">
#    <authorgroup>
#    <author><surname>Behera</surname> <firstname>S. K.</firstname> </author>
#    <author><firstname>J.</firstname> <surname>Luo</surname></author>
#    <author><firstname>S.</firstname> <surname>Masson</surname></author>
#    <author><firstname>S.</firstname> <surname>Rao</surname></author>
#    <author><firstname>S.</firstname> <surname>Gualdi</surname></author>
#    <author><firstname>P.</firstname> <surname>Delecluse</surname></author>
#    <author><firstname>A.</firstname> <surname>Navara</surname></author>
#    <author><firstname>T.</firstname> <surname>Yamagata</surname></author>
#    </authorgroup>
#    <date>2004</date>
#    <title>Paramount Impact of the Indian Ocean Dipole on the East African
#    Short Rains: A CGCM Study</title>
#    <publishername>J. Climate</publishername>
#    <biblioid class="doi">doi</biblioid>
#    <bibliomisc role="pseudoref">In press.</bibliomisc>
#    <bibliomisc role="internalref">from
#    http://www.lodyc.jussieu.fr/~opatlod/NEMO_v1/6_Menu/2_page/index.html
#    2007-03-29T16:24:31Z fplod by hand</bibliomisc>
#    </biblioentry>
#
#   @book{MetReiCoh2004 ,
#      author    = {Michael Metcalf and
#              John Reid and Malcolm Cohen},
#      title     = {Fortran 95/2003 explained},
#      year      = {2004},
#      publisher = {Oxford University Press},
#      edition   = {Third},
#      ISBN      = {0-19-852693-8},
#      pages = {434}
#   }
# donnerait
#    <biblioentry xml:id="MetReiCoh2004">
#    <authorgroup>
#    <author><firstname>Michael</firstname> <surname>Metcalf</surname></author>
#    <author><firstname>John</firstname> <surname>Reid</surname></author>
#    <author><firstname>Malcolm</firstname> <surname>Cohen</surname></author>
#    </authorgroup>
#    <date>2004}</date>
#    <title>Fortran 95/2003 explained</title>
#    <publisher><publishername>Oxford University Press</publishername></publisher>
#    <biblioid class="isbn">0-19-852693-8</biblioid>
#    <pagenums>434</pagenums>
#    <edition>Third</edition>
#
# example :
# $ ./bibopa.sh -p birefnemo -i data/biball.txt -t raw
# $ ./bibopa.sh -p bibrefnemo -i data/mail2007-05-10T09:01:56Z -t mailbody
# $ ./bibopa.sh -p polyfortran -i data/petitpolyfp.bib -t bibtex -l fr
# $ ./bibopa.sh -p demo1 -i data/bibdemo1.txt -t raw -l fr
#
# see also mailtouser.sh
#
# original location :
# /usr/home/fplod/incas/bibnemo/src/bibnemomaf/bibopa.sh sur cerbere.locean-ipsl.upmc.fr
#
# update
# ++ gestion des comments
# ++ gestion des id existants (cf  la fin)
# ++ option debug
# $Id$
# fplod 2008-04-30T07:11:58Z aedon.locean-ipsl.upmc.fr (Darwin)
# chgt for dbk5 out
# xmlto 0.0.18 does'nt like this docbook release. remove
# usage of xml (xmlstarlet) for validation
# bug fix
# add journal names from demo1
# fplod 2008-03-12T16:11:07Z aedon.locean-ipsl.upmc.fr (Darwin)
# add -p option (to overide bibrefnemo in file name and id)
# add -l option by default en
# fplod 2008-03-11T11:25:27Z aedon.locean-ipsl.upmc.fr (Darwin)
# add bibtex entry (not finished)++
# fplod 2007-06-20T17:18:02Z aedon.locean-ipsl.upmc.fr (Darwin)
# <bibliomisc role="id"> replace by <biblioid class="doi">
# smasson 2007-06-07T16:43:42Z arete.locean-ipsl.upmc.fr (Darwin)
# Add journals
# fplod 2007-05-10T09:17:09Z aedon.locean-ipsl.upmc.fr (Darwin)
# dernires touches
# fplod 2007-04-25T10:59:49Z cerbere.locean-ipsl.upmc.fr (Linux)
# add a filein parameter and an option for mailbody
# comments  (line begininig with #) are now possible
# Sebastien Masson avril 2007 creation
#

rmbl () {
    echo "${1}" | sed -e "s/^ *//" | sed -e "s/ *$//"
}
cleanname () {
    echo "${1}" | sed -e "s/^ *//" \
	-e "s/^ *,//" \
	-e "s/^ *;//" \
	-e "s/^ *\.//" \
	-e "s/ *$//" \
	-e "s/, *$//" \
	-e "s/; *$//" \
	-e "s/\. *$//"
}
#
set -o posix
command=$(basename ${0})
log_date=$(date -u +"%Y-%m-%dT%H:%M:%SZ")
log=/tmp/${command}.${log_date}
#
# test if xsltproc is available
type xsltproc 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
 echo " eee : xsltproc not found"
 exit 1
fi
#
# test if xml  is available
type xml 1> /dev/null 2>&1
status=${?}
if [ ${status} -ne 0 ]
then
 echo " eee : xml not found"
 exit 1
fi
#
usage=" Usage : ${command} -i filein -t type -p project -l lang"
#
# default
lang=en
#
minargcount=6
#echo " narg ${#}"
if [ ${#} -lt ${minargcount} ]
then
   echo "eee : not enought arguments"
   echo "${usage}"
   exit 1
fi
#
while [ ! -z "${1}" ]
do
 case ${1} in
 -i) # filein
  filein=${2}
  shift
 ;;
 -t) # type
  type=${2}
  shift
 ;;
 -p)
  project=${2}
  shift
 ;;
 -l)
  lang=${2}
  shift
 ;;
 -h)
  echo "${usage}"
  exit 0
 ;;
 *) # other choice
  echo "${usage}"
  exit 1
 ;;
 esac
 shift # next flag
done
#
set -u
#
# check for filein
if [ ! -f ${filein} ]
then
   echo "eee : ${filein} not found"
   exit 1
fi
#
case ${type} in
raw) # file like data/biball.txt
 fileraw=${filein}
;;
mailbody) # file like data/mail2007-04-25T08:58:16Z.txt
 fileraw=/tmp/$(basename ${filein}).raw
 echo "# from ${filein}" > ${fileraw}
 echo "# ${log_date}" >> ${fileraw}
 echo "# corrections" >> ${fileraw}
 grep "correction_.*=" ${filein} | grep -v "correction_.*=$" | \
 sed -e "s/correction_.*=//">> ${fileraw}
 echo "# new references" >> ${fileraw}
 awk "/newreferences=/,/comments=/" ${filein} | sed -e "s/newreferences=//" -e "/comments=/d" >> ${fileraw}
 #more ${fileraw} # ++ if debug
 #read a #++ if debug
;;
bibtex) # file like data/petitpoly.bib
 fileraw=/tmp/$(basename ${filein}).raw
 awk -f join_endcomma.awk ${filein} >> ${fileraw}
 #more ${fileraw} # ++ if debug
 #read a #++ if debug
;;
*)
   echo "eee : type should be raw, mailbody or bibtex"
   exit 1
;;
esac
#
# check for output
fileou=$( basename ${filein} .txt).xml
if [ -f ${fileou} ]
then
   echo "eee : ${fileou} already exist"
   exit 1
fi
#
jlist[1]="J. Climate"
jlist[2]="Journal of Climate"
jlist[3]="Ocean Modelling"
jlist[4]="Geophys. Res. Lett."
jlist[5]="J. Geophys. Res."
jlist[6]="Tellus A"
jlist[7]="Tellus B"
jlist[8]="J. Phys. Oceanogr."
jlist[9]="Clim. Dyn."
jlist[10]="Climate Dynamics"
jlist[11]="Dyn. Atmos. Oceans"
jlist[12]="Mon. Wea. Rev."
jlist[13]="Global Biogeochem. Cycles"
jlist[14]="Nonlinear Processes in Geophysics"
jlist[15]="Ocean Science"
jlist[16]="J. Mar. Systems"
jlist[17]="J. Atmos. Sc."
jlist[18]="Proc. Royal Soc."
jlist[19]="Bull. Amer. Meteorol. Soc."
jlist[20]="Ocean Dyn."
jlist[21]="Geophysical Monograph Series"
jlist[22]="Paleoceanography"
jlist[23]="Annales Geophysicae"
jlist[24]="Annals of Geophys."
jlist[25]="Deep Sea Research Part II: Topical Studies in Oceanography"
jlist[26]="Deep Sea Res. II"
jlist[27]="Atmospheric Chemistry and Physics"
jlist[28]="Atmospheric Chemistry and Physics Discussions"
jlist[29]="Earth Plan. Sc. Lett."
jlist[30]="J. Mar. Res."
jlist[31]="PCMDI Report Series"
jlist[32]="J. Mar. Syst."
jlist[33]="Note du Pole de Modelisation"
jlist[34]="Calculateurs Paralleles"
jlist[35]="Note Scientifique du Pole de Modelisation"
jlist[36]="Lecture Notes in Computer Science"
jlist[37]="Prog. Oceanogr."
jlist[38]="Deep Sea Research Part I: Oceanographic Research Papers"
jlist[39]="Deep Sea Res. I"
jlist[40]="IEEE Trans. Geosci. Remote Sensing"
jlist[41]="ECMWF Tech. Memorandum"
jlist[42]="J. Environ. Radioactivity"
jlist[43]="Int. WOCE Newsletter"
jlist[44]="C. R. Acad. Sci. Paris"
jlist[45]="C. R. Acad. Sci. Paris, Earth and Planetary Sciences"
jlist[46]="In The mathematics of models for climatology and environment"
jlist[47]="Oceanol. Acta"
jlist[48]="The global atmosphere and ocean system"
jlist[49]="NATO Advanced Study Institute"
jlist[50]="WCRP"
jlist[51]="The Courier"
jlist[52]="Elsevier Oceanographic Series"
jlist[53]="In Conference Proceedings of the 1988 International Conference on Supercomputing"
jlist[54]="In Science and engineering on Cray Supercomputers"
jlist[55]="In Modeling the Earth's Climate and its Variability"
jlist[56]="Fish. Oceanogr."
jlist[57]="Q. J. R. Meteor. Soc."
jlist[58]="In High performance computing in the geosciences"
jlist[59]="Int. J. Numer. Meth. in Fluids"
jlist[60]="Lecture notes in Physics"
jlist[61]="J. Meterol. Soc. Japan"
jlist[62]="Journal of Marine Systems"
jlist[63]="Canadian Journal of Fisheries and Aquatic Sciences"
jlist[64]="J. Atmos. Ocean. Tech."
jlist[65]="EOS"
jlist[66]="Nature"
jlist[67]="Physics and Chemistry of the Earth, Part B: Hydrology, Oceans and Atmosphere"
jlist[68]="Journal of Atmospheric and Oceanic Technology"
jlist[69]="Lecture Notes in Computational Science Engineering"
jlist[70]="In Greenhouse Gas Control Technologies"
jlist[71]="Chemical Geology"
jlist[72]="Marine Geology"
jlist[73]="Developments in Paleoenvironmental Research"
jlist[74]="Science"
jlist[75]="In Mediterranean Climate Variability"
jlist[76]="Global and Planetary Change"
jlist[77]="Physical Review Letters"
#
# following Journal name are fake ones for demo1 : references extracted from
# Cantratrix Sopronica L. written by Georges Perec
#find in http://www-rocq.inria.fr/who/Marc.Thiriet/PsFil/Biblio/cantatrix.ps
ij=77
ij=$(( ${ij} + 1))
jlist[$ij]="Res. Proc. neurophysiol. Fanatic Soc"
ij=$(( ${ij} + 1))
jlist[$ij]="New Records Ass. J."
ij=$(( ${ij} + 1))
jlist[$ij]="Am. J. Allegrol."
ij=$(( ${ij} + 1))
jlist[$ij]="J. Neurochem. Neurocytol. Enzymol."
ij=$(( ${ij} + 1))
jlist[$ij]="Hammersmith J."
ij=$(( ${ij} + 1))
jlist[$ij]="Nasa Rept."
ij=$(( ${ij} + 1))
jlist[$ij]="C.r. Assoc. Conc. Lam."
ij=$(( ${ij} + 1))
jlist[$ij]="Gaz. med. franco-rus."
ij=$(( ${ij} + 1))
jlist[$ij]="Amer. J. music. Deciency"
#jlist[]=""
#
#
    cat <<EOF > ${fileou}
<?xml version='1.0' encoding='ISO-8859-1'?>
<bibliography
version="5.0"
xmlns="http://docbook.org/ns/docbook"
xml:id="bib${project}ref"
xml:lang="${lang}">

<info>
<date>$( date -u +"%Y-%m-%dT%H:%M:%SZ" )</date>
</info>
EOF
#
# strip comments
fileraw_strict=/tmp/$(basename ${fileraw})_strict
grep -v "^#" ${fileraw} > ${fileraw_strict}
totlines=$( wc -l ${fileraw_strict} | awk '{print $1}' )
l=1
while [ ${l} -le ${totlines}  ]
do
# extract one line
  line=$( sed -n ${l}p ${fileraw_strict} )
  orgline=$( echo ${line} | sed -e "s/--/- -/g" )
  line=$( echo ${line} | sed -e "s/</\&lt;/g" -e "s/>/\&gt;/g" )
  # detect if bibtex or not bibtex (starting with @)
  if [ ${line:0:1} != "@" ]
  then
     # parsing non bibtex line
     bibtex=0
     # before the first :
       tmp=${line%%:*}
     # before the last ,
       auths=${tmp%,*},
     # replace "and" by ","
       auths=$( echo "${auths}" | sed -e "s/ and /,/g" )
     # after the last ,
       year=${tmp##*,}
       year=$( rmbl "${year}" )
     ## first author before the first .,
       first=${auths%%.,*}.
     # its firstname after the last ,
       firstfn=${first##*,}
       firstfn=$( rmbl "${firstfn}" )
     # its surname ; before the first ,
       firstsn=${first%%,*}
       firstsn=$( rmbl "${firstsn}" )
     ## ref id
       refid=$( echo ${firstsn} | tr "[:upper:]" "[:lower:]" | tr -s " " "_"  | tr -s "'" "_" | recode -d -f ISO-8859-1..flat )${year}
       num=$( grep -c "<biblioentry xml:id=\"${refid}_[0-9][0-9]\">" ${fileou} )
       num=$(( ${num} + 1 ))
       [ ${num} -le 9 ] && num=0${num}
       refid=${refid}_${num}
       hasauthor=1
   else
      # parsing bibtex line
      bibtex=1
      # before the first ,
      tmp=${line%%,*}
      endline=${line#*,}
      # refid is after { and before the first comma
      refid=${tmp#*{}
      # ++ test if refid already exist
      # check if author field is there
      echo ${endline} | grep -q "author"
      hasauthor=${?}
      if [ ${hasauthor} -eq 0 ]
      then
         auths=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(author * = *{\)\(.*\)\(}\)/\2/")
         #echo "auths $auths" # genre  Michael Metcalf and John Reid and Malcolm Cohen
      else
        echo "pas d'auteurs" # ++
      fi
      endline=${endline#*,}
      title=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(title * = *{\)\(.*\)\(}\)/\2/")
      endline=${endline#*,}
#echo ${endline} | awk -F "," '{print $1}' #++debug
      year=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(year * = *{\)\(.*\)\(}\)/\2/")
      endline=${endline#*,}
      publisher=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(publisher * = *{\)\(.*\)\(}\)/\2/")
      endline=${endline#*,}
      edition=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(edition * = *{\)\(.*\)\(}\)/\2/")
      endline=${endline#*,}
      isbn=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(isbn * = *{\)\(.*\)\(}\)/\2/")
      endline=${endline#*,}
      pag=$( echo ${endline} | awk -F "," '{print $1}' | sed -e "s/\(page * = *{\)\(.*\)\(}\)/\2/")

      if [ ${hasauthor} -eq 1 ]
      then
         ## first author before the first and
         first=${auths%%and*}
         # its surname after the last " "  ++ pas vrai double nom et pb blanc dans la syntaxe
         # firstsn=${first##* } . ok en interactif . pas ok en script
         firstsn=$( echo ${first}  | awk '{print $2}')
         #echo "firstsn $firstsn"
         # its firstname before the first " "
         firstfn=${first%% *}
      fi
   fi

	  cat <<EOF >> ${fileou}
<biblioentry xml:id="${refid}">
  <!-- date
  $( date -u +"%Y-%m-%dT%H:%M:%SZ" )
  -->
  <!-- original text
  ${orgline}
  -->
EOF
      if [ ${hasauthor} -eq 1 ]
      then
	  cat <<EOF >> ${fileou}
  <authorgroup>
    <author> <personname> <surname>${firstsn}</surname> <firstname>${firstfn}</firstname> </personname> </author>
EOF

## other authors..
  if [ ${bibtex} -eq 0 ]
  then
   previous=${first},
   # next authors...
   next=${auths##*${previous}}
  fi
  if [ ${bibtex} -eq 1 ]
  then
   previous=${first}"and "
   # next authors...
   #echo "auths ${auths}"
   next=${auths##*${previous}}
   if [ "${next}and " = "${previous}" ]
   then
     next=""
   fi
  fi
  #echo "previous ${previous}"
  #echo "next ${next}"
# while the next author is not empty
  while [  "${next}" != "" ]
    do
      if [ ${bibtex} -eq 0 ]
      then
         # get the first next author; before the first ,
         next=${next%%,*}
         # its surname ; after the last .
         nextsn=${next##*.}
         nextsn=$( rmbl "${nextsn}" )
         # its firstname ; before the last .
         nextfn=${next%.*}.
         nextfn=$( rmbl "${nextfn}" )
       fi
      if [ ${bibtex} -eq 1 ]
      then
         # get the first next author; before the first "and"
         next=${next%%and *}
         # its surname after the last " "  ++ pas vrai double nom et pb blanc dans la syntaxe
         # nextsn=${next##* } . ok en interactif . pas ok en script
         nextsn=$( echo ${next}  | awk '{print $2}')
         #echo "nextsn $nextsn"
         # its nextname before the first " "
         nextfn=${next%% *}
      fi
#
    echo "    <author> <personname> <surname>${nextsn}</surname> <firstname>${nextfn}</firstname> </personname> </author>" >> ${fileou}
    #echo "    <author> <personname> <surname>${nextsn}</surname> <firstname>${nextfn}</firstname> </personname> </author>"  #++debug
    if [ ${bibtex} -eq 0 ]
    then
      previous=${next},
      next=${auths##*${previous}}
    fi
    if [ ${bibtex} -eq 1 ]
    then
      previous=${next}"and "
      next=${auths##*${previous}}
    fi
  done
  echo "  </authorgroup>"  >> ${fileou}
  fi

# end of the line ; after the first :
  endline=${line#*:}

## find the journal
  j=1
  jfound=""
  jlistsize=${#jlist[@]}
  while [[ ${j} -le ${jlistsize} && "${jfound}" == "" ]]
    do
    ok=$( echo ${endline} | grep -ci "${jlist[j]} *," )
    [ $ok -eq 1 ] && jfound="${jlist[j]}"
    j=$(( ${j} + 1 ))
  done
  if [ "${jfound}" == "" ]
      then
      echo "eee: Journal not found "
      echo "${endline}"
      #++exit 1
#+++ following lines are not yet validate
      # it might be a book, a manual, a conference, etc. ++
      echo "  <title>${title}</title>" >> ${fileou}
      echo "  <biblioid class=\"isbn\">${isbn}</biblioid>" >> ${fileou}
	  cat <<EOF >> ${fileou}
  <biblioset relation="nojournal">
    <title>${title}</title>
    <pagenums>${pag}</pagenums>
    <pubdate>${year}</pubdate>
  </biblioset>
EOF
	  cat <<EOF >> ${fileou}
</biblioentry>

EOF
  fi # end of if jfound empty (ie not an article)
  if [ "${jfound}" != "" ]
  then
## title
# before the first :
  title=${endline%%${jfound}*}
  title=$( cleanname "${title}" )
  echo "  <title>${title}</title>" >> ${fileou}
## end
## end of the line ; after the first ${jfound}
  endline=${endline#*${jfound}}
  endline=$( cleanname "${endline}" )
## doi
  endline=$( echo ${endline} | sed -e "s/[dD][oO][iI] *\t* *: *\t* */doi:/" )
  ok=$( echo ${endline} | grep -ic "doi:" )
  if [ ${ok} -eq 1 ]
      then
      doi=${endline##*doi:}
      echo "  <biblioid class=\"doi\">${doi}</biblioid>" >> ${fileou}
      endline=${endline%doi:*}
      endline=$( cleanname "${endline}" )
  else
      echo "non doi: ${line}"
  fi
  num=$( echo ${endline} |  tr -dc "," | wc -c )
  case ${num} in
      1)
### echo ${num}: ${endline}
	  vol=${endline%,*}
	  vol=$( cleanname "${vol}" )
	  pag=${endline##*,}
	  pag=$( cleanname "${pag}" )
	  cat <<EOF >> ${fileou}
  <biblioset relation="journal">
    <title>${jfound}</title>
    <volumenum>${vol}</volumenum><pagenums>${pag}</pagenums>
    <pubdate>${year}</pubdate>
  </biblioset>
EOF
      ;;
      2)
	  vol=${endline%,*}
	  vol=$( cleanname "${vol}" )
	  iss=${vol##*,}
	  iss=$( cleanname "${iss}" )
	  vol=${vol%,*}
	  vol=$( cleanname "${vol}" )
	  pag=${endline##*,}
	  pag=$( cleanname "${pag}" )
	  cat <<EOF >> ${fileou}
  <biblioset role="journal">
    <title>${jfound}</title>
    <volumenum>${vol}</volumenum><issuenum>${iss}</issuenum><pagenums>${pag}</pagenums>
    <pubdate>${year}</pubdate>
  </biblioset>
EOF
      ;;
      *)
#echo ${num}: ${endline} # ++debug
	  cat <<EOF >> ${fileou}
  <biblioset role="journal">
    <title>${jfound}</title>
    <pubdate>${year}</pubdate>
    <bibliomisc>${endline}</bibliomisc>
  </biblioset>
EOF
      ;;
  esac

	  cat <<EOF >> ${fileou}
</biblioentry>

EOF




  fi # end of if jfound not empty
  l=$(( ${l} + 1 ))

done
echo "</bibliography>" >> ${fileou}

xml val --err \
        --xsd http://www.docbook.org/xml/5.0/xsd/docbook.xsd \
	${fileou} 1> xmlstarlet.log 2>&1

# clean
echo "iii : xslstarlet.log contains sdtout and stderr from xml command on ${fileou}"
echo "iii : which was done just to check consistence of ${fileou}"
more xmlstarlet.log
rm -i xmlstarlet.log
#
case ${type} in
raw)
 echo "iii : ${fileraw_strict} contains a copy of input file without comments"
 rm -i ${fileraw_strict}
;;
mailbody)
 echo "iii : ${fileraw} contains a copy of input file"
 rm -i ${fileraw}
 echo "iii : ${fileraw_strict} contains a copy of input file without comments"
 rm -i ${fileraw_strict}
;;
esac
#
echo " iii : you have a some new or modified bibliographic references in ${fileou} (biblioentry)"
echo " iii : you can add them in ${fileou}" #++ filename path
echo " iii : modify date in ${fileou}" #++ filename path
echo " iii : but please take care of id last part (after underscore) that may"
echo " iii : be in conflict with existing ones" #++ pour s'affranchir de ce souci il faudrait donner en entre complmentaire le fichier  complter.
#
exit 0
